import { createCanvas, registerFont } from "canvas";
import dateFormat from "dateformat";
import { lineNotify } from "./lineNotify";
import { sqlSendQuery } from "./sql";

// 生成图片的函数
export function generateImage(props: ICanvas) {
	const { date, team, rounds = 3 } = props;
	const formatDate = `${date.slice(0, 4)}-${date.slice(4, 6)}-${date.slice(6, 8)}`;
	const { aTeam, bTeam } = splitTeams(team);

	if (!aTeam || !bTeam) {
		return "";
	}
	const person = aTeam.length > bTeam.length ? aTeam.length : bTeam.length;
	if (aTeam.length > bTeam.length) {
		bTeam.push("那個")
	} else if (aTeam.length < bTeam.length) {
		aTeam.push("那個")
	}
	const canvasWidth = 800; // 根据需要调整宽度
	const canvasHeight = (rounds * 100) + (person * 10); // 根据需要调整高度
	const canvas = createCanvas(canvasWidth, canvasHeight);
	const ctx = canvas.getContext("2d");


	// 注册自定义字体
	const fontPath = "./src/fonts/PMingLiU.ttf";
	registerFont(fontPath, { family: "PMingLiU" });
	const font = "PMingLiU";

	// 设定背景颜色
	ctx.fillStyle = "white";
	ctx.fillRect(0, 0, canvas.width, canvas.height);

	// 追踪已使用的组合
	let usedCombinations = new Set<string>();

	// 生成不重复的队伍组合并确保每人上场
	function generateTeams(): { teamA: string, teamB: string }[] {
		const roundTeams: { teamA: string, teamB: string }[] = [];
		const availableBoys = [...aTeam];
		const availableGirls = [...bTeam];
		let attempts = 0;

		while (roundTeams.length < aTeam.length) {
			const boyIndex = Math.floor(Math.random() * availableBoys.length);
			const girlIndex = Math.floor(Math.random() * availableGirls.length);
			const team = `${availableBoys[boyIndex]}-${availableGirls[girlIndex]}`;

			// 如果组合未使用过，添加到队伍列表
			if (!usedCombinations.has(team)) {
				roundTeams.push({
					teamA: availableBoys[boyIndex],
					teamB: availableGirls[girlIndex]
				});
				usedCombinations.add(team);
				availableBoys.splice(boyIndex, 1);
				availableGirls.splice(girlIndex, 1);
			}

			// 如果尝试过多次未找到合适组合，强制生成剩余队伍
			attempts++;
			if (attempts > 10 && roundTeams.length < aTeam.length) {
				for (let i = 0; i < availableBoys.length; i++) {
					roundTeams.push({
						teamA: availableBoys[i],
						teamB: availableGirls[i % availableGirls.length]
					});
				}
				break;
			}
		}

		return roundTeams;
	}

	// 生成所有轮次的队伍组合
	let teamCombinations: { teamA: string, teamB: string }[][] = [];
	for (let i = 0; i < rounds; i++) {
		const newRound = generateTeams();
		teamCombinations.push(newRound);

		// 如果所有组合都用过了，重置组合记录
		if (usedCombinations.size >= aTeam.length * bTeam.length) {
			usedCombinations.clear();
		}
	}

	// 绘制标题和日期
	ctx.fillStyle = "black";
	ctx.font = `bold 24px "${font}"`;
	ctx.textAlign = "center";
	ctx.fillText(`${formatDate} 比賽隊伍`, canvasWidth / 2, 40);

	const roundTitleYStart = 80;
	const teamYStart = 60;
	const lineHeight = 30;
	const roundsPerRow = 3;
	const rowHeight = 300;

	const result = {};
	teamCombinations.forEach((roundTeams, roundIndex) => {
		result[roundIndex] = []
		const rowIndex = Math.floor(roundIndex / roundsPerRow);
		const colIndex = roundIndex % roundsPerRow;
		const roundXPosition = (canvasWidth / roundsPerRow) * colIndex + (canvasWidth / roundsPerRow) / 2;
		const roundYPosition = roundTitleYStart + rowIndex * rowHeight;

		ctx.font = `bold 18px "${font}"`;
		ctx.textAlign = "center";
		ctx.fillText(`第 ${roundIndex + 1} 輪`, roundXPosition, roundYPosition);

		ctx.font = `16px "${font}"`;

		roundTeams.forEach((team, teamIndex) => {
			result[roundIndex].push([team.teamA, team.teamB])
			const teamYPosition = roundYPosition + teamYStart + teamIndex * lineHeight;
			if (teamIndex === 0) {
				ctx.fillText("1號隊友", roundXPosition - 50, teamYPosition);
				ctx.fillText("2號隊友", roundXPosition + 50, teamYPosition);
			}
			ctx.fillText(team.teamA, roundXPosition - 50, teamYPosition + lineHeight);
			ctx.fillText(team.teamB, roundXPosition + 50, teamYPosition + lineHeight);
		});
	});

	// 导出图片
	const buffer = canvas.toBuffer("image/png");
	// fs.writeFileSync("./team_combination.png", buffer);
	let dateTime: string = dateFormat(new Date(), "yyyy-mm-dd HH:MM:ss");
	console.log(`${dateTime} [canvas] 圖片已生成！`);


	const personnelStr: string = JSON.stringify(team);
	const resultStr: string = JSON.stringify(result);
	const query: string = String.Format("INSERT INTO `badminton` (time,personnel,battlecombination) VALUES ({0},'{1}','{2}') ON DUPLICATE KEY UPDATE personnel='{1}',battlecombination='{2}';"
		, date, personnelStr, resultStr);
	sqlSendQuery(query);

	const message = `\n${formatDate} 比賽隊伍`;
	lineNotify({
		LINE_ACCESS_TOKEN: process.env.LINE_ACCESS_TOKEN,
		message,
		imageFile: buffer
	})
	return buffer;
}

interface ICanvas {
	date: string;
	team: any;
	rounds: number;
}

function splitTeams(data: [number, string][]): { aTeam: string[], bTeam: string[] } {
	const aTeam: string[] = [];
	const bTeam: string[] = [];

	// 分离 boys 和 girls
	data.forEach(([type, name]) => {
		if (type === 1) {
			aTeam.push(name);
		} else if (type === 0) {
			bTeam.push(name);
		}
	});

	// 进行平衡
	while (aTeam.length > bTeam.length + 1) {
		bTeam.push(aTeam.pop()!); // 移动一名男生到女生
	}

	// 平衡后可能还有空缺
	const missingBoys = aTeam.length;
	const missingGirls = bTeam.length;
	const balanceSize = Math.max(missingBoys, missingGirls);

	if (missingBoys > balanceSize) {
		// 若男生过多，添加补充人员到女生
		for (let i = 0; i < missingBoys; i++) {
			bTeam.push("那個");
		}
	} else if (missingGirls > balanceSize) {
		// 若女生过多，添加补充人员到男生
		for (let i = 0; i < missingGirls; i++) {
			aTeam.push("那個");
		}
	}

	return { aTeam, bTeam };
}